<?php
class Ssc_Service_SeoScore
{
    const FACTOR_NAMESPACE = 'SeoScore_Factor_';
    const HELPER_NAMESPACE = 'SeoScore_Helper_';
    const META_NAME = 'factors';

    protected $_factorNames = array(
        'TitleContain',
        'TitleBegin',
        'TitleLength',
        'UrlContain',
        'DescriptionContain',
        'DescriptionBegin',
        'DescriptionLength',
        'KeywordsContain',
        'H1Contain',
        'H1Begin',
        'H2Contain',
        'H3Contain',
        'KeywordDensityLimits',
        'WordsFirst',
        'ImgAltContain',
        'DecorationBold',
        'DecorationItalic',
        'DecorationUnderlined',
        'LinkExternal',
        'LinkInternal',
        'WordsLast',

        'KeywordDensity',
        'Score',
    );
    protected $_factors;
    protected $_factorsByType;
    protected $_helpers = array();
    protected $_postId;
    protected $_data = array();

    protected $_analyzeData;

    protected $_commonWords;

    public function __construct()
    {
        $this->_loadFactors();
    }

    public function analyze($data)
    {
        $this->setPostId($data['post']->ID);

        $data['secondary'] = '' != $data['secondary']
            ? explode("\n", $data['secondary'])
            : array();

        $data['html'] = $this->filterHtml(
            Ssc_Application::getService('PostHtml')->get($data['post']));
        $data['url'] =
            Ssc_Application::getService('PostUrl')->get($data['post']);

        $data['post_html'] = $this->filterPost(
            Ssc_Application::getService('PostContent')
                ->get($data['post'])
        );
        $data['post_text'] = $this->htmlToText($data['post_html']);

        $this->_analyzeData = $data;
        unset($data);

        foreach ($this->_factors as $factor) {
            $factor->analyze($this);
        }

        return $this;
    }

    public function load()
    {
        $meta = Ssc_Application::getModel('PostMeta')->setPostId($this->_postId);
        $data = (array) $meta->getValue(self::META_NAME);
        $extraData = isset($data['_extra'])
            ? (array) $data['_extra']
            : array();

        foreach ($this->_factors as $name=>$factor) {
            if ($metaName = $factor->getMetaName()) {
                $factor->setValue($meta->getValue($metaName));
            } else {
                $factor->setValue(isset($data[$name])? $data[$name] : NULL);
            }

            if (isset($data['_extra'][$name])) {
                $factor->setExtraData((array) $data['_extra'][$name]);
            }
        }

        return $this;
    }

    public function save()
    {
        $meta = Ssc_Application::getModel('PostMeta')->setPostId($this->_postId);
        $data = array('_extra' => array());

        foreach ($this->_factors as $name=>$factor) {
            $value = $factor->getValue();
            if ($metaName = $factor->getMetaName()) {
                $meta->setValue($metaName, $value);
            } else if (!is_null($value)) {
                $data[$name] = $value;
            }

            if ($extra = $factor->getExtraData()) {
                $data['_extra'][$name] = $extra;
            }
        }

        $meta->setValue(self::META_NAME, $data);

        return $this;
    }

    public function getFormattedData($type = NULL, $skipNull = FALSE)
    {
        $factors = $this->getFactors($type);

        $result = array();
        foreach ($factors as $name=>$factor) {
            if (!$factor->getVisible()
                || ($skipNull && is_null($factor->getValue()))
            ) {
                continue;
            }

            $result[$factor->getLabel()] = $factor->getFormattedValue();
        }

        return $result;
    }

    protected function _loadFactors()
    {
        $this->_factorsByType = array(
            'bool' => array(),
            'numeric' => array(),
        );

        foreach ($this->_factorNames as $name) {
            $factor = Ssc_Application::getService(
                self::FACTOR_NAMESPACE . $name);
            $this->_factors[$name] = $factor;
            $this->_factorsByType[$factor->getFactorType()][$name] = $factor;
        }

        return $this;
    }

    public function getHelper($name)
    {
        $name = ucfirst($name);

        if (!isset($this->_helpers[$name])) {
            $this->_helpers[$name] = Ssc_Application::getService(
                self::HELPER_NAMESPACE . $name);
        }

        return $this->_helpers[$name];
    }

    public function __call($method, $arguments)
    {
        $helper = $this->getHelper($method);
        array_unshift($arguments, $this);

        return call_user_func_array(array($helper, 'direct'), $arguments);
    }

    /**
    * Helper methods
    */

    public function filterHtml($html)
    {
        //Remove comments
        $this->eraseFragments($html, '<!--', '-->');

        //Remove scripts
        $html = preg_replace('~<\s*script\s*>.*?<\s*/\s*script\s*>~is', '', $html);

        //Normalize spaces and whitespace
        $this->massReplace(array("\n", "\r", "\n", "\t", '&nbsp;', '&#160;'), /*, chr(194) . chr(160), chr(160)) */
            ' ', $html);

        //Remove control chars
        $this->massReplace(
            array(
                chr(0), chr(1), chr(2), chr(3), chr(4), chr(5), chr(6), chr(7),
                chr(8), chr(9), chr(10), chr(11), chr(12), chr(13), chr(14),
                chr(15), chr(16), chr(17), chr(18), chr(19), chr(20), chr(21),
                chr(22), chr(23), chr(24), chr(25), chr(26), chr(27), chr(28),
                chr(29), chr(30), chr(31),

                chr(194) . chr(128), chr(194) . chr(129), chr(194) . chr(130),
                chr(194) . chr(131), chr(194) . chr(132), chr(194) . chr(133),
                chr(194) . chr(134), chr(194) . chr(135), chr(194) . chr(136),
                chr(194) . chr(137), chr(194) . chr(138), chr(194) . chr(139),
                chr(194) . chr(140), chr(194) . chr(141), chr(194) . chr(142),
                chr(194) . chr(143), chr(194) . chr(144), chr(194) . chr(145),
                chr(194) . chr(146), chr(194) . chr(147), chr(194) . chr(148),
                chr(194) . chr(149), chr(194) . chr(150), chr(194) . chr(151),
                chr(194) . chr(152), chr(194) . chr(153), chr(194) . chr(154),
                chr(194) . chr(155), chr(194) . chr(156), chr(194) . chr(157),
                chr(194) . chr(158), chr(194) . chr(159),
            ),
            '', $html);

        return $html;
    }

    public function filterPost($html)
    {
        $html = strip_shortcodes($html);
        $html = $this->filterHtml($html);

        return $html;
    }

    public function massReplace(array $search, $replace, &$subject)
    {
        foreach ($search as $s) {
            $subject = str_replace($s, $replace, $subject);
        }

        return $this;
    }

    public function eraseFragments(&$subject, $start, $end)
    {
        for ($i=0; $i<200; $i++) {
            $posStart = strpos($subject, $start);
            if ($posStart===FALSE) break;
            $pos = $posStart+strlen($start);
            if ($pos >= strlen($subject)) {
                break;
            }
            $posEnd = strpos($subject, $end, $pos);
            if ($posEnd===FALSE) break;

            $subject = substr($subject, 0, $posStart) . substr($subject, $posEnd + strlen($end));
        }

        return $this;
    }

    public function getCommonWords()
    {
        if (!$this->_commonWords) {
            $h = fopen(SSC_APPLICATION_PATH . '/etc/common-words.txt', 'r');
            while (!feof ($h)) {
               $line = trim(fgets($h, 4096));
               if ('' == $line) {
                   continue;
               }
               $this->_commonWords[] = $line;
            }
            fclose ($h);
        }

        return $this->_commonWords;
    }

    public function removeCommonWords($text)
    {
        foreach($this->getCommonWords() as $word) {
            $text = preg_replace('~\b' . preg_quote($word, '~') . '\b~i', '', $text);
        }
        return trim($text);
    }

    /**
    * Accessors
    */

    public function getFactorNames()
    {
        return $this->_factorNames;
    }

    public function getFactors($type = NULL)
    {
        return $type
            ? $this->_factorsByType[$type]
            : $this->_factors;
    }

    public function setPostId($value)
    {
        $this->_postId = $value;
        return $this;
    }

    public function getPostId()
    {
        return $this->_postId;
    }

    public function getAnalyzeData($key = NULL)
    {
        return $key? $this->_analyzeData[$key] : $this->_analyzeData;
    }
}
